package com.muyu.task.server.thread;

import java.util.concurrent.*;
import java.util.concurrent.atomic.AtomicInteger;

public class OptimizedPrioritizedThreadPool {

    private static final ExecutorService executor;
    private static final Semaphore highPrioritySemaphore;
    private static final Semaphore mediumPrioritySemaphore;
    private static final Semaphore lowPrioritySemaphore;
    private static final Semaphore emergencySemaphore;

    private static final int totalThreads =20; // 固定线程池大小
    private static final int defaultHighThreads = 10;
    private static final int defaultMediumThreads = 6;
    private static final int defaultLowThreads = 4;

    private static final int emergencyThreads = 9;
    private static final int emergencyHighThreads = 3;
    private static final int emergencyMediumThreads = 3;
    private static final int emergencyLowThreads = 3;

    private static volatile boolean inEmergencyMode = false;
    public static final AtomicInteger activeEmergencyTasks = new AtomicInteger(0); // 紧急任务计数器
    public static final AtomicInteger remainingTasks = new AtomicInteger(0); // 剩余任务计数器
    private static final CountDownLatch emergencyLatch = new CountDownLatch(emergencyThreads); // 用于等待所有紧急任务完成

    static {
        if (defaultHighThreads + defaultMediumThreads + defaultLowThreads > totalThreads) {
            throw new IllegalArgumentException("Default priority threads exceed total threads.");
        }
        if (emergencyThreads + emergencyHighThreads + emergencyMediumThreads + emergencyLowThreads > totalThreads) {
            throw new IllegalArgumentException("Emergency priority threads exceed total threads.");
        }

        // 创建固定大小的线程池
        executor = new ThreadPoolExecutor(
                totalThreads, totalThreads,
                80L, TimeUnit.SECONDS,
                new LinkedBlockingQueue<Runnable>()
        );
        highPrioritySemaphore = new Semaphore(defaultHighThreads);
        mediumPrioritySemaphore = new Semaphore(defaultMediumThreads);
        lowPrioritySemaphore = new Semaphore(defaultLowThreads);
        emergencySemaphore = new Semaphore(0);
    }

    public static void submitEmergencyTask(Runnable task) {
        if (activeEmergencyTasks.incrementAndGet() == 1) {
            // 初始化计数器
            remainingTasks.set(0);
            adjustForEmergency();
        }
        remainingTasks.incrementAndGet(); // 增加剩余任务计数
        emergencyLatch.countDown(); // 减少计数器
        try {
            emergencySemaphore.acquire();
            executor.submit(() -> {
                try {
                    task.run();
                } finally {
                    emergencySemaphore.release();
                    if (remainingTasks.decrementAndGet() == 0) {
                        try {
                            Thread.sleep(800000);
                        } catch (InterruptedException e) {
                            throw new RuntimeException(e);
                        }
                        restoreDefaults();
                    }
                }
            });
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
        }
    }

    public static void submitHighPriorityTask(Runnable task) {
        try {
            highPrioritySemaphore.acquire();
            executor.submit(() -> {
                try {
                    task.run();
                } finally {
                    highPrioritySemaphore.release();
                }
            });
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
        }
    }

    public static void submitMediumPriorityTask(Runnable task) {
        try {
            mediumPrioritySemaphore.acquire();
            executor.submit(() -> {
                try {
                    task.run();
                } finally {
                    mediumPrioritySemaphore.release();
                }
            });
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
        }
    }

    public static void submitLowPriorityTask(Runnable task) {
        try {
            lowPrioritySemaphore.acquire();
            executor.submit(() -> {
                try {
                    task.run();
                } finally {
                    lowPrioritySemaphore.release();
                }
            });
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
        }
    }

    public static synchronized void adjustForEmergency() {
        if (!inEmergencyMode) {
            inEmergencyMode = true;

            // 释放一些高、中、低优先级的线程，以便为紧急任务腾出空间
            highPrioritySemaphore.release(defaultHighThreads - emergencyHighThreads);
            mediumPrioritySemaphore.release(defaultMediumThreads - emergencyMediumThreads);
            lowPrioritySemaphore.release(defaultLowThreads - emergencyLowThreads);

            // 为紧急任务分配线程
            emergencySemaphore.release(emergencyThreads);
        }
    }

    public static synchronized void restoreDefaults() {
        if (inEmergencyMode && remainingTasks.get() == 0) {
            // 清空紧急任务的许可
            emergencySemaphore.drainPermits();

            // 重新分配高、中、低优先级的线程
            try {
                highPrioritySemaphore.acquire(emergencyHighThreads);
                mediumPrioritySemaphore.acquire(emergencyMediumThreads);
                lowPrioritySemaphore.acquire(emergencyLowThreads);
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }

            highPrioritySemaphore.release(defaultHighThreads);
            mediumPrioritySemaphore.release(defaultMediumThreads);
            lowPrioritySemaphore.release(defaultLowThreads);

            // 清空紧急任务的许可
            emergencySemaphore.release(0);

            // 退出紧急模式
            inEmergencyMode = false;

            // 确保所有紧急任务都完成了
            try {
                emergencyLatch.await(); // 等待所有紧急任务完成
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                System.err.println("Interrupted while waiting for emergency tasks to complete.");
            }
        }
    }

    // 移除shutdown方法
}
